TitleSelect = {}
TitleSelect.Locale = {}

TitleSelect.VERSION_STR = "X"
TitleSelect.AUTHOR = "DOOMED"

TitleSelect.Locale.ADDON_NAME = "DoomedTitleSelect"
TitleSelect.Locale.DESCRIPTION = "Short-list selection of character titles"

TitleSelect.Locale.REMOVE_FROM_LIST = "Remove from list"
TitleSelect.Locale.FORMAT_TITLE_ID_LOST = TitleSelect.Locale.ADDON_NAME..":  Lost title ID %d"
TitleSelect.Locale.ERROR_HOOK_DROP_DOWN = TitleSelect.Locale.ADDON_NAME.." error:  cannot hook drop-down menu"
TitleSelect.Locale.SHOW_SELECTED_ICON = "Show selected title icon"
TitleSelect.Locale.FRAME_POS_LOCKED = "Frame position locked"
TitleSelect.Locale.ICON_SIZE = "Icon size"
TitleSelect.Locale.TOOLTIP_STYLE_STR = "Title tooltip style:"
TitleSelect.Locale.TOOLTIP_STYLES = {
	["none"] = "None",
	["abbreviated"] = "Abbreviated",
	["full"] = "Full"
}
TitleSelect.Locale.TOOLTIP_HIDE_SINGLE_LINE = "Hide single-line tooltips"
TitleSelect.Locale.CENTER_BUTTON = "Center"
TitleSelect.Locale.CENTER_BUTTON_TOOLTIP = "Reposition frame to center"
TitleSelect.Locale.PREFERENCES_HEADER = "Changes to preferences take effect upon restart."
TitleSelect.Locale.PREFERRED_INTERFACE = "Preferred interface:"
TitleSelect.Locale.PREFERRED_CONFIG_MANAGER = "Preferred configuration manager:"
TitleSelect.Locale.XBAR_BUTTON_TIP = "Left-click to select title"

TitleSelectSaves = {}
TitleSelectProfile = {}


local defaultConfig = {
	frameLocked = false,
	showSelectedTitleIcon = true,
	selectedTitleIconSize = 32,
	tooltipStyle = "abbreviated",
	tooltipHideSingleLine = true,
	prefInterface = "XBar",
	prefConfigManager = "XBar",
}

TitleSelectProfile.titleIds = {}
TitleSelect.shortlistInfo = {}

TitleSelect.currentId = nil

TitleSelect.COMBOBOX_TEXT_WIDTH_OFFSET = 20
local RESIZE_BUTTON_WIDTH = 8
local ICON_INSET = 3
local TIP_STRINGS_NUM = 60


function TitleSelect.OutputText(msg, red, green, blue)
	DEFAULT_CHAT_FRAME:AddMessage(msg, red, green, blue)
end


function TitleSelect.Frame_OnLoad(frame)
	SaveVariables("TitleSelectSaves")
	SaveVariablesPerCharacter("TitleSelectProfile")

	frame:RegisterEvent("VARIABLES_LOADED")
	frame:RegisterEvent("PLAYER_TITLE_ID_CHANGED")

	TitleSelect.ReanchorCorner(frame)

	TitleSelect.HookDropDownMenu()

	UIDropDownMenu_SetWidth(TitleSelectFrame_ComboBox, frame:GetWidth() - RESIZE_BUTTON_WIDTH - TitleSelect.COMBOBOX_TEXT_WIDTH_OFFSET)
	UIDropDownMenu_Initialize(TitleSelectFrame_ComboBox, TitleSelect.CB_CallbackShow)
end


function TitleSelect.Frame_OnEvent(frame, event, arg1)
	if (event == "VARIABLES_LOADED") then
		TitleSelect.VariablesLoaded()
	elseif (event == "PLAYER_TITLE_ID_CHANGED") then
		TitleSelect.UpdateCurrentTitle()
	end
end


function TitleSelect.VariablesLoaded()
	local k, v

	for k, v in pairs(defaultConfig) do
		if (TitleSelectSaves[k] == nil) then
			TitleSelectSaves[k] = v
		end
	end

	TitleSelect.ShowFrame(TitleSelectFrame_IconContainer, TitleSelectSaves.showSelectedTitleIcon)
	TitleSelect.ShowFrame(TitleSelectFrame_SizeButton, not TitleSelectSaves.frameLocked)

	TitleSelect.SetIconContainerSize(TitleSelectSaves.selectedTitleIconSize)

	local i = 1
	local info
	while (TitleSelectProfile.titleIds[i]) do
		info = TitleSelect.GetTitleData(TitleSelectProfile.titleIds[i])
		if (info) and (info.acquired) then
			table.insert(TitleSelect.shortlistInfo, {["name"] = info.name, ["icon"] = info.icon})
			i = i + 1
		else
			TitleSelect.OutputText(string.format(TitleSelect.Locale.FORMAT_TITLE_ID_LOST, TitleSelectProfile.titleIds[i]), 0.5, 0.5, 0.5)
			table.remove(TitleSelectProfile.titleIds, i)
		end
	end

	if (TitleSelectSaves.anchor) then
		local anc = TitleSelectSaves.anchor
		TitleSelectFrame:ClearAllAnchors()
		TitleSelectFrame:SetAnchor(anc.point, anc.relativePoint, anc.relativeTo, anc.x, anc.y)
	end
	if (TitleSelectSaves.iconContainerAnchor) then
		local anc = TitleSelectSaves.iconContainerAnchor
		TitleSelectFrame_IconContainer:ClearAllAnchors()
		TitleSelectFrame_IconContainer:SetAnchor(anc.point, anc.relativePoint, anc.relativeTo, anc.x, anc.y)
	end
	if (TitleSelectSaves.cbWidth) then
		TitleSelectFrame:SetWidth(TitleSelectSaves.cbWidth + RESIZE_BUTTON_WIDTH)
	end

	TitleSelect.SnapAnchorInit(TitleSelectFrame, 1)

	TitleSelect.InitInterfaceSelection()
	TitleSelect.InitConfigManagerSelection()

	if (TitleSelectSaves.prefInterface == "XBar") then
		TitleSelect.XBarInterface_CreateButton()
	end
	if (TitleSelectSaves.prefConfigManager == "XBar") then
		TitleSelect.XBarInterface_AttachConfig()
	end

	TitleSelect.UpdateCurrentTitle()

	TitleSelect.AddonManagerRegister()
end


function TitleSelect.Frame_OnSizeChanged(frame)
	UIDropDownMenu_SetWidth(TitleSelectFrame_ComboBox, frame:GetWidth() - RESIZE_BUTTON_WIDTH - TitleSelect.COMBOBOX_TEXT_WIDTH_OFFSET)
end


function TitleSelect.ReanchorCorner(frame)
	local x, y = frame:GetPos()
	frame:ClearAllAnchors()
	frame:SetAnchor("TOPLEFT", "TOPLEFT", UIParent, x, y)
end


function TitleSelect.ShowFrame(frame, show)
	if (show) then
		frame:Show()
	else
		frame:Hide()
	end
end


function TitleSelect.SetIconContainerSize(iconSize)
	local newSize = iconSize + 2*ICON_INSET
	TitleSelectFrame_IconContainer:SetSize(newSize, newSize)
end


function TitleSelect.Frame_OnMouseDown(frame)
	if (not TitleSelectSaves.frameLocked) then
		TitleSelect.SnapAnchorStartMoving(frame)
	end
end


function TitleSelect.Frame_OnMouseUp(frame)
	TitleSelect.SnapAnchorStopMoving(frame)

	TitleSelectSaves.anchor = {}
	TitleSelect.FrameToSavableAnchor(frame, TitleSelectSaves.anchor)
end


function TitleSelect.FrameToSavableAnchor(frame, sAnchor)
	sAnchor.point, sAnchor.relativePoint, sAnchor.relativeTo, sAnchor.x, sAnchor.y = frame:GetAnchor()
	if not (sAnchor.relativeTo) then
		sAnchor.relativeTo = frame:GetParent()
	end
	sAnchor.relativeTo = sAnchor.relativeTo:GetName()
end


function TitleSelect.SizeButton_OnMouseDown(frame)
	TitleSelectFrame:StartSizing("RIGHT")
end


function TitleSelect.SizeButton_OnMouseUp(frame)
	TitleSelectFrame:StopMovingOrSizing()
	TitleSelectSaves.cbWidth = TitleSelectFrame:GetWidth() - RESIZE_BUTTON_WIDTH
end


function TitleSelect.CB_CallbackShow()
	local i, v
	local buttonInfo

	if (UIDROPDOWNMENU_MENU_LEVEL == 1) then
		for i, v in ipairs(TitleSelect.shortlistInfo) do
			buttonInfo = {}
			buttonInfo.text = v.name
			buttonInfo.value = TitleSelectProfile.titleIds[i]
			buttonInfo.func = TitleSelect.CB_CallbackDropDownClickTitle
			if (TitleSelect.currentId == TitleSelectProfile.titleIds[i]) then
				buttonInfo.checked = 1
			end
			UIDropDownMenu_AddButton(buttonInfo, 1)
		end
	elseif (UIDROPDOWNMENU_MENU_LEVEL == 2) then
		buttonInfo = {}
		buttonInfo.text = TitleSelect.Locale.REMOVE_FROM_LIST
		buttonInfo.func = TitleSelect.CB_CallbackDropDownClickRemove
		buttonInfo.notCheckable = 1
		UIDropDownMenu_AddButton(buttonInfo, 2)
	end
end


function TitleSelect.CB_CallbackDropDownClickTitle(button)
	TitleSelect.SetTitle(button.value)
end


function TitleSelect.CB_CallbackDropDownClickRemove(button)
	local slIndex
	slIndex = getglobal(UIDROPDOWNMENU_MENU_BUTTON):GetID()

	TitleSelect.RemoveTitle(slIndex)
	CloseDropDownMenus()
end


function TitleSelect.UpdateSelection(slIndex)
	TitleSelectFrame_ComboBoxText:SetText(TitleSelect.shortlistInfo[slIndex].name)

	if (TitleSelect.shortlistInfo[slIndex].icon) then
		TitleSelectFrame_IconContainer_Texture:SetTexture(TitleSelect.shortlistInfo[slIndex].icon)
		TitleSelectFrame_IconContainer_Texture:Show()
	else
		TitleSelectFrame_IconContainer_Texture:Hide()
	end

	TitleSelect.XBarInterface_UpdateSelection(slIndex)
end


function TitleSelect.SetTitle(id)
	SetTitleRequest(id)
end


function TitleSelect.FindShortlistIndex(id)
	local i, v

	for i, v in ipairs(TitleSelectProfile.titleIds) do
		if (id == v) then
			return i
		end
	end

	return 0
end


function TitleSelect.UpdateCurrentTitle()
	local id
	local slIndex

	id = GetCurrentTitle()
	slIndex = TitleSelect.FindShortlistIndex(id)

	if (slIndex == 0) then
		TitleSelect.AddTitle(id)
		slIndex = #(TitleSelectProfile.titleIds)
	end

	TitleSelect.currentId = id
	TitleSelect.UpdateSelection(slIndex)
end


function TitleSelect.AddTitle(tid)
	local info

	info = TitleSelect.GetTitleData(tid)
	if (info) and (info.acquired) then
			table.insert(TitleSelectProfile.titleIds, tid)
		table.insert(TitleSelect.shortlistInfo, {["name"] = info.name, ["icon"] = info.icon})
	end
end


function TitleSelect.RemoveTitle(slIndex)
	table.remove(TitleSelectProfile.titleIds, slIndex)
	table.remove(TitleSelect.shortlistInfo, slIndex)
end


function TitleSelect.GetTitleData(id)
	UpdateTitleInfo()

	local info
	local matchInfo
	local titleIndex
	local count = GetTitleCount()

	info = {}
	matchInfo = {}

	if (id == DF_CA_HT_CUSPMIZE) then
		matchInfo.name = GetCusomizeTitle()
		matchInfo.titleID = id
		matchInfo.acquired = 1
		matchInfo.icon = "interface/icons/quest_paperstack03"
		return matchInfo
	end
	if (id == 0) then
		matchInfo.name = C_TITLE_NIL
		matchInfo.titleID = 0
		matchInfo.acquired = 1
		matchInfo.icon = nil
		return matchInfo
	end

	for i = 1, count do
		info.name, 
		info.titleID, 
		info.acquired, 
		info.icon = GetTitleInfoByIndex(i - 1)

		if (id == info.titleID) then
			titleIndex = i - 1
			matchInfo.name       = info.name
			matchInfo.titleID    = info.titleID
			matchInfo.acquired   = info.acquired  -- nil or 1
			matchInfo.icon       = info.icon
			return matchInfo
		end
	end

	return nil
end


function TitleSelect.ShowTitleTooltip(frame)
	local tipframe = TitleSelectFrame_Tooltip
	local r, g, b
	local i
	local fontStr
	local colorIndex

	if (frame.value < 1) then
		return
	end

	GameTooltip:SetOwner(frame, "ANCHOR_RIGHT", 16, -4)

	if (TitleSelectSaves.tooltipStyle == "abbreviated") then
		tipframe:ClearLines()
		for i = 1, TIP_STRINGS_NUM do
			tipframe:AddLine()
		end

		tipframe:SetTitle(frame.value)
		tipframe:Hide()

		fontStr = getglobal(tipframe:GetName().."TextLeft1")
		r, g, b = fontStr:GetColor()
		GameTooltip:SetText(fontStr:GetText(), r, g, b)

		for i = 2, TIP_STRINGS_NUM do
			fontStr = getglobal(tipframe:GetName().."TextLeft"..i)
			r, g, b = fontStr:GetColor()
			colorIndex = string.find(fontStr:GetText(), "|c")

			if (r == 1 and g == 1 and b == 1) and (colorIndex ~= 1) then
				break
			end

			GameTooltip:AddLine(fontStr:GetText(), r, g, b)
		end

		if (TitleSelectSaves.tooltipHideSingleLine) and (TitleSelect.TooltipIsOneLine(GameTooltip)) then
			GameTooltip:Hide()
		else
			TitleSelect.MassageTooltipPosition(frame)
			GameTooltip:Show()
		end
	elseif (TitleSelectSaves.tooltipStyle == "full") then
		GameTooltip:SetTitle(frame.value)

		if (TitleSelectSaves.tooltipHideSingleLine) and (TitleSelect.TooltipIsOneLine(GameTooltip)) then
			GameTooltip:Hide()
		else
			TitleSelect.MassageTooltipPosition(frame)
		end
	end
end


function TitleSelect.TooltipIsOneLine(tipframe)
	local i
	for i = 2, TIP_STRINGS_NUM do
		if (getglobal(tipframe:GetName().."TextLeft"..i):IsVisible()) then
			return false
		end
	end
	return true
end


function TitleSelect.MassageTooltipPosition(frame)
	local scrW = GetScreenWidth()
	local scrH = GetScreenHeight()
	local point = "LEFT"
	local relPoint = "RIGHT"
	local adjX = frame:GetParent():GetRight() - frame:GetRight()
	local tipW = GameTooltip:GetWidth()
	local tipH = GameTooltip:GetHeight()
	local cY = frame:GetTop() + (frame:GetHeight() / 2)
	local adjY = 0

	if (frame:GetRight() + adjX + tipW > scrW) then
		point = "RIGHT"
		relPoint = "LEFT"
		adjX = frame:GetParent():GetLeft() - frame:GetLeft()
	end

	if (cY + tipH/2 > scrH) then
		adjY = scrH - tipH/2 - cY
	end
	if (cY - tipH/2 < 0) then
		adjY = tipH/2 - cY
	end

	GameTooltip:ClearAllAnchors()
	GameTooltip:SetAnchor(point, relPoint, frame, adjX, adjY)
end

function TitleSelect.SnapAnchorInit(frame, isRoot)
	frame.limitBorder = 1
	UIPanelt_AutoAnchor(frame, "TitleSelectGroup", isRoot)
end


function TitleSelect.SnapAnchorStartMoving(frame)
	AutoAnchorManager:Hide()

	local x,y = GetCursorPos()
	local _x = frame:GetLeft()
	local _y = frame:GetTop()

	g_AutoAnchorFrame = frame
	g_AutoAnchorFrame_DeltaX = _x - x
	g_AutoAnchorFrame_DeltaY = _y - y
	AutoAnchorManager:Show()
end


function TitleSelect.SnapAnchorStopMoving(frame)
	AutoAnchorManager:Hide()
end


function TitleSelect.IconContainer_OnLoad(frame)
	TitleSelect.SnapAnchorInit(frame, nil)
end


function TitleSelect.IconContainer_OnMouseDown(frame)
	if (not TitleSelectSaves.frameLocked) then
		TitleSelect.SnapAnchorStartMoving(frame)
	end
end


function TitleSelect.IconContainer_OnMouseUp(frame)
	TitleSelect.SnapAnchorStopMoving(frame)

	TitleSelectSaves.iconContainerAnchor = {}
	TitleSelect.FrameToSavableAnchor(frame, TitleSelectSaves.iconContainerAnchor)
end