
local interfaces = {
	XBar = {available = true},
	Standard = {available = true},
}

local managers = {
	XBar = {available = true},
	Standard = {available = true},
}

function TitleSelect.ConfigContainer_OnLoad(frame)
	TitleSelectConfigContainer:SetBackdropTileColor(0, 0, 0)
	TitleSelectConfigContainer:SetBackdropTileAlpha(0.85)

	TitleSelectConfigContainer_Title:SetText(TitleSelect.Locale.ADDON_NAME)
	TitleSelectConfigContainer_Version:SetText(TitleSelect.VERSION_STR)
end


function TitleSelect.ConfigContainer_OnShow(frame)
	TitleSelectConfigContainer:ClearAllAnchors()
	TitleSelectConfigContainer:SetAnchor("CENTER", "CENTER", UIParent, 0, 0)

	TitleSelectConfig:ClearAllAnchors()
	TitleSelectConfig:SetAnchor("TOP", "TOP", TitleSelectConfigContainer, 0, 30)
	TitleSelectConfig:Show()

	local w, h = TitleSelectConfig:GetSize()
	TitleSelectConfigContainer:SetSize(w, h + 30)
end

function TitleSelect.ConfigContainer_OnHide(frame)
	TitleSelectConfig:Hide()
end


function TitleSelect.ConfigContainer_OnMouseDown(frame)
	frame:StartMoving("CENTER")
end


function TitleSelect.ConfigContainer_OnMouseUp(frame)
	frame:StopMovingOrSizing()
end


function TitleSelect.Config_OnLoad(frame)
	TitleSelectConfig_ShowSelectedIconCheckbox_Text:SetText(TitleSelect.Locale.SHOW_SELECTED_ICON)
	TitleSelectConfig_FrameLockCheckbox_Text:SetText(TitleSelect.Locale.FRAME_POS_LOCKED)
	TitleSelectConfig_TooltipHideSingleLineCheckbox_Text:SetText(TitleSelect.Locale.TOOLTIP_HIDE_SINGLE_LINE)

	local initialSize = TitleSelectFrame_IconContainer_Texture:GetWidth()
	local min, max = 20, 32
	TitleSelectConfig_IconSizeDesc:SetText(TitleSelect.Locale.ICON_SIZE)
	TitleSelectConfig_IconSize:SetValueStepMode("INT")
	TitleSelectConfig_IconSize:SetMinMaxValues(min, max)
	TitleSelectConfig_IconSizeLow:SetText(tostring(min))
	TitleSelectConfig_IconSizeHigh:SetText(tostring(max))
	TitleSelectConfig_IconSize:SetValue(initialSize)

	TitleSelectConfig_TooltipStyleStr_Text:SetText(TitleSelect.Locale.TOOLTIP_STYLE_STR)

	UIDropDownMenu_SetWidth(TitleSelectConfig_TooltipStyleComboBox, TitleSelectConfig_TooltipStyleComboBox:GetWidth() - TitleSelect.COMBOBOX_TEXT_WIDTH_OFFSET)
	UIDropDownMenu_Initialize(TitleSelectConfig_TooltipStyleComboBox, TitleSelect.ConfigTooltipStyleCB_CallbackShow)

	TitleSelectConfig_CenterButton:SetText(TitleSelect.Locale.CENTER_BUTTON)

	TitleSelectConfig_PreferencesHeaderStr_Text:SetText(TitleSelect.Locale.PREFERENCES_HEADER)

	TitleSelectConfig_InterfaceStr_Text:SetText(TitleSelect.Locale.PREFERRED_INTERFACE)
	UIDropDownMenu_SetWidth(TitleSelectConfig_InterfaceComboBox, 110)
	UIDropDownMenu_Initialize(TitleSelectConfig_InterfaceComboBox, TitleSelect.ConfigInterfaceCB_CallbackShow)

	TitleSelectConfig_ConfigManagerStr_Text:SetText(TitleSelect.Locale.PREFERRED_CONFIG_MANAGER)
	UIDropDownMenu_SetWidth(TitleSelectConfig_ConfigManagerComboBox, 110)
	UIDropDownMenu_Initialize(TitleSelectConfig_ConfigManagerComboBox, TitleSelect.ConfigManagerCB_CallbackShow)
end


function TitleSelect.Config_OnShow(frame)
	TitleSelectConfig_ShowSelectedIconCheckbox:SetChecked(TitleSelectSaves.showSelectedTitleIcon)
	TitleSelectConfig_FrameLockCheckbox:SetChecked(TitleSelectSaves.frameLocked)
	TitleSelectConfig_TooltipHideSingleLineCheckbox:SetChecked(TitleSelectSaves.tooltipHideSingleLine)

	TitleSelectConfig_IconSize:SetValue(TitleSelectSaves.selectedTitleIconSize)

	TitleSelectConfig_TooltipStyleComboBoxText:SetText(TitleSelect.Locale.TOOLTIP_STYLES[TitleSelectSaves.tooltipStyle])

	TitleSelect.UpdateInterfaceCB()
	TitleSelect.UpdateConfigManagerCB()
end

function TitleSelect.ConfigShowSelectedIcon_OnClick(frame)
	TitleSelectSaves.showSelectedTitleIcon = frame:IsChecked()
	TitleSelect.ShowFrame(TitleSelectFrame_IconContainer, TitleSelectSaves.showSelectedTitleIcon)
end


function TitleSelect.ConfigFrameLock_OnClick(frame)
	TitleSelectSaves.frameLocked = frame:IsChecked()
	TitleSelect.ShowFrame(TitleSelectFrame_SizeButton, not TitleSelectSaves.frameLocked)
end


function TitleSelect.ConfigTooltipHideSingleLine_OnClick(frame)
	TitleSelectSaves.tooltipHideSingleLine = frame:IsChecked()
end


function TitleSelect.ConfigIconSize_OnValueChanged(frame)
	GCF_Slider_OnValueChanged(frame)

	TitleSelectSaves.selectedTitleIconSize = frame:GetValue()
	TitleSelect.SetIconContainerSize(TitleSelectSaves.selectedTitleIconSize)
end


function TitleSelect.ConfigTooltipStyleCB_CallbackShow()
	local k, v
	local buttonInfo

	for k, v in pairs(TitleSelect.Locale.TOOLTIP_STYLES) do
		buttonInfo = {}
		buttonInfo.text = v
		buttonInfo.value = k
		buttonInfo.func = TitleSelect.ConfigTooltipStyleCB_CallbackDropDownClick
		if (TitleSelectSaves.tooltipStyle == k) then
			buttonInfo.checked = 1
		end
		UIDropDownMenu_AddButton(buttonInfo, 1)
	end
end


function TitleSelect.ConfigTooltipStyleCB_CallbackDropDownClick(button)
	TitleSelectConfig_TooltipStyleComboBoxText:SetText(button:GetText())
	TitleSelectSaves.tooltipStyle = button.value
end


function TitleSelect.ConfigCenter_OnClick(frame)
	TitleSelectFrame:ClearAllAnchors()
	TitleSelectFrame:SetAnchor("CENTER", "CENTER", UIParent, 0, 0)

	TitleSelect.ReanchorCorner(TitleSelectFrame)

	local point, relativePoint, relativeTo = TitleSelectFrame_IconContainer:GetAnchor()
	if (relativeTo ~= TitleSelectFrame) then
		TitleSelectFrame_IconContainer:ClearAllAnchors()
		TitleSelectFrame_IconContainer:SetAnchor("BOTTOMRIGHT", "BOTTOMLEFT", TitleSelectFrame, 0, 0)
	end

	TitleSelectSaves.anchor = {}
	TitleSelect.FrameToSavableAnchor(TitleSelectFrame, TitleSelectSaves.anchor)
	TitleSelectSaves.iconContainerAnchor = {}
	TitleSelect.FrameToSavableAnchor(TitleSelectFrame_IconContainer, TitleSelectSaves.iconContainerAnchor)
end

function TitleSelect.ConfigCenter_OnEnter(frame)
	GameTooltip:SetOwner(frame, "ANCHOR_BOTTOM", 0, 2)
	GameTooltip:SetText(TitleSelect.Locale.CENTER_BUTTON_TOOLTIP, 1, 1, 1)
	GameTooltip:Show()
end

function TitleSelect.ConfigCenter_OnLeave(frame)
	GameTooltip:Hide()
end


function TitleSelect.InitInterfaceSelection()
	if (interfaces[TitleSelectSaves.prefInterface] == nil) then
		TitleSelectSaves.prefInterface = "Standard"
	end

	interfaces.XBar.available = TitleSelect.XBarInterface_Available()
end

function TitleSelect.UpdateInterfaceCB()
	TitleSelectConfig_InterfaceComboBoxText:SetText(TitleSelectSaves.prefInterface)
	if (interfaces[TitleSelectSaves.prefInterface].available) then
		TitleSelectConfig_InterfaceComboBoxText:SetColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
	else
		TitleSelectConfig_InterfaceComboBoxText:SetColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b)
	end
end


function TitleSelect.ConfigInterfaceCB_CallbackShow()
	local k, v
	local buttonInfo
	local curInterface = TitleSelectSaves.prefInterface

	for k, v in pairs(interfaces) do
		buttonInfo = {}
		buttonInfo.text = k
		buttonInfo.value = k
		if (not v.available) then
			buttonInfo.textR = GRAY_FONT_COLOR.r
			buttonInfo.textG = GRAY_FONT_COLOR.g
			buttonInfo.textB = GRAY_FONT_COLOR.b
		end
		buttonInfo.func = TitleSelect.ConfigInterfaceCB_CallbackDropDownClick
		if (curInterface == k) then
			buttonInfo.checked = 1
		end
		UIDropDownMenu_AddButton(buttonInfo, 1)
	end
end

function TitleSelect.ConfigInterfaceCB_CallbackDropDownClick(button)
	TitleSelectSaves.prefInterface = button.value

	TitleSelect.UpdateInterfaceCB()
end


function TitleSelect.InitConfigManagerSelection()
	if (managers[TitleSelectSaves.prefConfigManager] == nil) then
		TitleSelectSaves.prefConfigManager = "Standard"
	end

	managers.XBar.available = TitleSelect.XBarInterface_Available()
end

function TitleSelect.UpdateConfigManagerCB()
	TitleSelectConfig_ConfigManagerComboBoxText:SetText(TitleSelectSaves.prefConfigManager)
	if (managers[TitleSelectSaves.prefConfigManager].available) then
		TitleSelectConfig_ConfigManagerComboBoxText:SetColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
	else
		TitleSelectConfig_ConfigManagerComboBoxText:SetColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b)
	end
end


function TitleSelect.ConfigManagerCB_CallbackShow()
	local k, v
	local buttonInfo
	local curManager = TitleSelectSaves.prefConfigManager

	for k, v in pairs(managers) do
		buttonInfo = {}
		buttonInfo.text = k
		buttonInfo.value = k
		if (not v.available) then
			buttonInfo.textR = GRAY_FONT_COLOR.r
			buttonInfo.textG = GRAY_FONT_COLOR.g
			buttonInfo.textB = GRAY_FONT_COLOR.b
		end
		buttonInfo.func = TitleSelect.ConfigManagerCB_CallbackDropDownClick
		if (curManager == k) then
			buttonInfo.checked = 1
		end
		UIDropDownMenu_AddButton(buttonInfo, 1)
	end
end

function TitleSelect.ConfigManagerCB_CallbackDropDownClick(button)
	TitleSelectSaves.prefConfigManager = button.value

	TitleSelect.UpdateConfigManagerCB()
end
