
local xbarButtonCreated = false
local xbarConfigCreated = false

function TitleSelect.XBarInterface_Available()
	if (XBARVERSION) and (XBARVERSION >= 1.51) then
		return true
	end
	return false
end

function TitleSelect.XBarInterface_CreateButton()
	if (not TitleSelect.XBarInterface_Available()) then
		return
	end

	local addon = {
		btn = {
			name = "Ext_TitleSelect",  
			texture = "Interface/Buttons/QuickSlot-Red",  
		},
	}

	XBarExt_TitleSelect_ComboBoxLeft:Hide()
	XBarExt_TitleSelect_ComboBoxRight:Hide()
	XBarExt_TitleSelect_ComboBoxMiddle:Hide()
	XBarExt_TitleSelect_ComboBoxText:Hide()
	XBarExt_TitleSelect_ComboBoxButton:Hide()
	UIDropDownMenu_Initialize(XBarExt_TitleSelect_ComboBox, TitleSelect.CB_CallbackShow)

	if (XBSet.Ext_TitleSelect_X == nil) then
		XBSet.Ext_TitleSelect_X = 370
	end

	XAddon_Register(addon)

	CreateUIComponent("Texture", "TitleSelect_XBarButton_Highlight", "XBarExt_TitleSelect_B")
	TitleSelect_XBarButton_Highlight:SetSize(XBarExt_TitleSelect_B:GetSize())
	TitleSelect_XBarButton_Highlight:SetTexture("Interface/Buttons/PanelUpButton-Highlight")
	TitleSelect_XBarButton_Highlight:SetAlphaMode("ADD")
	TitleSelect_XBarButton_Highlight:Hide()
	XBarExt_TitleSelect_B:SetLayers(2, TitleSelect_XBarButton_Highlight)

	XBarExt_TitleSelect_B:SetScripts("OnClick", [=[ TitleSelect.XBarInterface_ButtonClick(this, key) ]=])
	XBarExt_TitleSelect_B:SetScripts("OnEnter", [=[ TitleSelect.XBarInterface_ButtonEnter(this) ]=])
	XBarExt_TitleSelect_B:SetScripts("OnLeave", [=[ TitleSelect.XBarInterface_ButtonLeave(this) ]=])

	XBarExt_TitleSelect_F_S:SetWidth(90)

	XBarExt_TitleSelect:Show()
	TitleSelectFrame_IconContainer:Hide()
	TitleSelectFrame:Hide()
	TitleSelectConfig_FrameLockCheckbox:Hide()
	TitleSelectConfig_ShowSelectedIconCheckbox:Hide()
	TitleSelectConfig_IconSize:Hide()
	TitleSelectConfig_CenterButton:Hide()

	xbarButtonCreated = true
end

function TitleSelect.XBarInterface_ButtonClick(frame, key, ...)
	if (key == "LBUTTON") then
		GameTooltip:Hide()
		XBarExt_TitleSelect_ComboBoxButton["__uiLuaOnClick__"](XBarExt_TitleSelect_ComboBoxButton, key, ...)
	end
end

function TitleSelect.XBarInterface_ButtonEnter(frame)
	TitleSelect_XBarButton_Highlight:Show()

	GameTooltip:SetOwner(frame, "ANCHOR_TOP", 0, 0)
	GameTooltip:SetText(TitleSelect.Locale.ADDON_NAME)
	GameTooltip:AddLine(TitleSelect.Locale.XBAR_BUTTON_TIP, 0, 0.75, 0.95)
end

function TitleSelect.XBarInterface_ButtonLeave(leave)
	TitleSelect_XBarButton_Highlight:Hide()
	GameTooltip:Hide()
end

function TitleSelect.XBarInterface_UpdateSelection(slIndex)
	if (xbarButtonCreated) then
		XBarExt_TitleSelect_F_S:SetText(TitleSelect.shortlistInfo[slIndex].name)

		if (TitleSelect.shortlistInfo[slIndex].icon) then
			XBarExt_TitleSelect_B_T:SetTexture(TitleSelect.shortlistInfo[slIndex].icon)
		else
			XBarExt_TitleSelect_B_T:SetTexture("Interface/Buttons/QuickSlot-Red")
		end
	end
end


function TitleSelect.XBarInterface_AttachConfig()
	if (not TitleSelect.XBarInterface_Available()) then
		return
	end

	local addon = {
		gui = {{
			name = TitleSelect.Locale.ADDON_NAME,  
			version = TitleSelect.VERSION_STR,  
			window = "TitleSelectConfig",  
		}},

			popup = {{
			icon = "Interface/Icons/quest_papyrus01",
			GetText = function() return TitleSelect.Locale.ADDON_NAME; end,
			OnClick = function(this, key) TitleSelect.XBarInterface_PopupClick(this, key); end,
		}},
	}

	XAddon_Register(addon)

	TitleSelectConfig:SetFrameStrata("MEDIUM")
	TitleSelectConfig:SetFrameLevel(1)
	XAddon_Page(TitleSelectConfig)

	TitleSelectMinimapButton:Hide()

	xbarConfigCreated = true
end

function TitleSelect.XBarInterface_PopupClick(frame, key)
	XAddon_ShowPage("TitleSelectConfig")
	if (XAddonMngr_Header_Name) then
 		XAddonMngr_Header_Name:SetText(TitleSelect.Locale.ADDON_NAME)
 	end
 	if (XAddonMngr_Header_Version) then
 		XAddonMngr_Header_Version:SetText(TitleSelect.VERSION_STR)
 	end
end
